@@ -40,6 +40,7 @@ import org.apache.cassandra.dht.IPartitioner;
 import org.apache.cassandra.exceptions.ConfigurationException;
 import org.apache.cassandra.exceptions.InvalidRequestException;
 import org.apache.cassandra.service.reads.SpeculativeRetryPolicy;
+import org.apache.cassandra.transport.ProtocolVersion;
 import org.apache.cassandra.utils.AbstractIterator;
 import org.github.jamm.Unmetered;
 
@@ -48,12 +49,7 @@ import static com.google.common.collect.Iterables.transform;
 import static java.lang.String.format;
 import static java.util.stream.Collectors.toList;
 import static java.util.stream.Collectors.toSet;
-import static org.apache.cassandra.db.Directories.SECONDARY_INDEX_NAME_SEPARATOR;
-import static org.apache.cassandra.db.Directories.TABLE_DIRECTORY_NAME_SEPARATOR;
-import static org.apache.cassandra.schema.KeyspaceMetadata.validateKeyspaceName;
-import static org.apache.cassandra.schema.SchemaConstants.FILENAME_LENGTH;
-import static org.apache.cassandra.schema.SchemaConstants.TABLE_NAME_LENGTH;
-import static org.apache.cassandra.schema.SchemaConstants.isValidCharsName;
+import static org.apache.cassandra.schema.IndexMetadata.isNameValid;
 
 @Unmetered
 public class TableMetadata implements SchemaElement
@@ -420,9 +416,11 @@ public class TableMetadata implements SchemaElement
 
     public void validate()
     {
-        validateKeyspaceName(keyspace, this::prepareConfigurationException);
+        if (!isNameValid(keyspace))
+            except("Keyspace name must not be empty, more than %s characters long, or contain non-alphanumeric-underscore characters (got \"%s\")", SchemaConstants.NAME_LENGTH, keyspace);
 
-        validateTableName();
+        if (!isNameValid(name))
+            except("Table name must not be empty, more than %s characters long, or contain non-alphanumeric-underscore characters (got \"%s\")", SchemaConstants.NAME_LENGTH, name);
 
         params.validate();
 
@@ -450,17 +448,6 @@ public class TableMetadata implements SchemaElement
         indexes.validate(this);
     }
 
-    private void validateTableName()
-    {
-        if (!isValidCharsName(name))
-            except("Table name must not be empty or not contain non-alphanumeric-underscore characters (got \"%s\")", name);
-
-        if (name.length() > TABLE_NAME_LENGTH)
-            except("Table name must not be more than %d characters long (got %d characters for \"%s\")", TABLE_NAME_LENGTH, name.length(), name);
-
-        assert getTableDirectoryName().length() <= FILENAME_LENGTH : String.format("Generated directory name for a table of %d characters doesn't fit the max filename legnth of %s. This unexpectedly wasn't prevented by check of the table name length, %d, to fit %d characters (got table name \"%s\" and generated directory name \"%s\"", getTableDirectoryName().length(), FILENAME_LENGTH, name.length(), TABLE_NAME_LENGTH, name, getTableDirectoryName());
-    }
-
     /**
      * To support backward compatibility with thrift super columns in the C* 3.0+ storage engine, we encode said super
      * columns as a CQL {@code map<blob, blob>}. To ensure the name of this map did not conflict with any other user
@@ -557,41 +544,7 @@ public class TableMetadata implements SchemaElement
     public String indexTableName(IndexMetadata info)
     {
         // TODO simplify this when info.index_name is guaranteed to be set
-        return name + SECONDARY_INDEX_NAME_SEPARATOR + info.name;
-    }
-
-    /**
-     * Returns the table part of the index table name or the entire table name
-     * if not an index table.
-     * @return table name part
-     */
-    public String getTableName()
-    {
-        int idx = name.indexOf(SECONDARY_INDEX_NAME_SEPARATOR);
-        return idx >= 0 ? name.substring(0, idx) : name;
-    }
-
-    /**
-     * Generates a directory name for the table by using table part of
-     * the (index) table name and table id.
-     * @return directory name
-     */
-    public String getTableDirectoryName()
-    {
-        return getTableName() + TABLE_DIRECTORY_NAME_SEPARATOR + id.toHexString();
-    }
-
-    /**
-     * Returns the index name from the name of an index table
-     * including the dot prefixing the index name, see {@link #indexTableName}.
-     * If not an index table, returns null.
-     * @return index name prefixed with dot prefix or null
-     */
-    @Nullable
-    public String getIndexNameWithDot()
-    {
-        int idx = name.indexOf(SECONDARY_INDEX_NAME_SEPARATOR);
-        return idx >= 0 ? name.substring(idx) : null;
+        return name + Directories.SECONDARY_INDEX_NAME_SEPARATOR + info.name;
     }
 
     /**
@@ -649,14 +602,9 @@ public class TableMetadata implements SchemaElement
         return builder.build();
     }
 
-    private ConfigurationException prepareConfigurationException(String format, Object... args)
-    {
-        return new ConfigurationException(keyspace + '.' + name + ": " + format(format, args));
-    }
-
     protected void except(String format, Object... args)
     {
-        throw prepareConfigurationException(format, args);
+        throw new ConfigurationException(keyspace + "." + name + ": " + format(format, args));
     }
 
     @Override
@@ -792,7 +740,12 @@ public class TableMetadata implements SchemaElement
                 partitioner = DatabaseDescriptor.getPartitioner();
 
             if (id == null)
-                id = TableId.generate();
+            {
+                // make sure vtables use determiniestic ids so they can be referenced in calls cross-nodes
+                // see CASSANDRA-17295
+                if (DatabaseDescriptor.useDeterministicTableID() || kind == Kind.VIRTUAL) id = TableId.unsafeDeterministic(keyspace, name);
+                else id = TableId.generate();
+            }
 
             if (Flag.isCQLTable(flags))
                 return new TableMetadata(this);
@@ -914,6 +867,13 @@ public class TableMetadata implements SchemaElement
             return this;
         }
 
+        public Builder memtable(MemtableParams val)
+        {
+            params.memtable(val);
+            return this;
+        }
+
+
         public Builder isCounter(boolean val)
         {
             return flag(Flag.COUNTER, val);
@@ -1037,6 +997,11 @@ public class TableMetadata implements SchemaElement
             return columns.values();
         }
 
+        public int numColumns()
+        {
+            return columns.size();
+        }
+
         public Set<String> columnNames()
         {
             return columns.values().stream().map(c -> c.name.toString()).collect(toSet());
@@ -1160,14 +1125,11 @@ public class TableMetadata implements SchemaElement
      */
     private static void addUserTypes(AbstractType<?> type, Set<ByteBuffer> types)
     {
-        AbstractType<?> unwrapped = type.unwrap();
+        // Reach into subtypes first, so that if the type is a UDT, it's dependencies are recreated first.
+        type.subTypes().forEach(t -> addUserTypes(t, types));
 
-        if (unwrapped.isUDT())
-        {
-            // Reach into subtypes first, so that if the type is a UDT, it's dependencies are recreated first.
-            unwrapped.subTypes().forEach(t -> addUserTypes(t, types));
-            types.add(((UserType)unwrapped).name);
-        }
+        if (type.isUDT())
+            types.add(((UserType)type).name);
     }
 
     @Override
@@ -1197,17 +1159,17 @@ public class TableMetadata implements SchemaElement
     }
 
     public String toCqlString(boolean includeDroppedColumns,
-                              boolean internals,
+                              boolean withInternals,
                               boolean ifNotExists)
     {
         CqlBuilder builder = new CqlBuilder(2048);
-        appendCqlTo(builder, includeDroppedColumns, internals, ifNotExists);
+        appendCqlTo(builder, includeDroppedColumns, withInternals, ifNotExists);
         return builder.toString();
     }
 
     public void appendCqlTo(CqlBuilder builder,
                             boolean includeDroppedColumns,
-                            boolean internals,
+                            boolean withInternals,
                             boolean ifNotExists)
     {
         assert !isView();
@@ -1246,7 +1208,7 @@ public class TableMetadata implements SchemaElement
         builder.append(" WITH ")
                .increaseIndent();
 
-        appendTableOptions(builder, internals);
+        appendTableOptions(builder, withInternals);
 
         builder.decreaseIndent();
 
@@ -1329,9 +1291,9 @@ public class TableMetadata implements SchemaElement
                .newLine();
     }
 
-    void appendTableOptions(CqlBuilder builder, boolean internals)
+    void appendTableOptions(CqlBuilder builder, boolean withInternals)
     {
-        if (internals)
+        if (withInternals)
             builder.append("ID = ")
                    .append(id.toString())
                    .newLine()
@@ -1388,6 +1350,66 @@ public class TableMetadata implements SchemaElement
         }
     }
 
+    /**
+     * Returns a string representation of a partition in a CQL-friendly format.
+     *
+     * For non-composite types it returns the result of {@link org.apache.cassandra.cql3.CQL3Type#toCQLLiteral}
+     * applied to the partition key.
+     * For composite types it applies {@link org.apache.cassandra.cql3.CQL3Type#toCQLLiteral} to each subkey and
+     * combines the results into a tuple.
+     *
+     * @param partitionKey a partition key
+     * @return CQL-like string representation of a partition key
+     */
+    public String partitionKeyAsCQLLiteral(ByteBuffer partitionKey)
+    {
+        return primaryKeyAsCQLLiteral(partitionKey, Clustering.EMPTY);
+    }
+
+    /**
+     * Returns a string representation of a primary key in a CQL-friendly format.
+     *
+     * @param partitionKey the partition key part of the primary key
+     * @param clustering the clustering key part of the primary key
+     * @return a CQL-like string representation of the specified primary key
+     */
+    public String primaryKeyAsCQLLiteral(ByteBuffer partitionKey, Clustering<?> clustering)
+    {
+        int clusteringSize = clustering.size();
+
+        String[] literals;
+        int i = 0;
+
+        if (partitionKeyType instanceof CompositeType)
+        {
+            List<AbstractType<?>> components = partitionKeyType.getComponents();
+            int size = components.size();
+            literals = new String[size + clusteringSize];
+            ByteBuffer[] values = ((CompositeType) partitionKeyType).split(partitionKey);
+            for (i = 0; i < size; i++)
+            {
+                literals[i] = asCQLLiteral(components.get(i), values[i]);
+            }
+        }
+        else
+        {
+            literals = new String[1 + clusteringSize];
+            literals[i++] = asCQLLiteral(partitionKeyType, partitionKey);
+        }
+
+        for (int j = 0; j < clusteringSize; j++)
+        {
+            literals[i++] = asCQLLiteral(clusteringColumns().get(j).type, clustering.bufferAt(j));
+        }
+
+        return i == 1 ? literals[0] : "(" + String.join(", ", literals) + ")";
+    }
+
+    private static String asCQLLiteral(AbstractType<?> type, ByteBuffer value)
+    {
+        return type.asCQL3Type().toCQLLiteral(value, ProtocolVersion.CURRENT);
+    }
+
     public static class CompactTableMetadata extends TableMetadata
     {
 
