@@ -24,6 +24,7 @@ import org.junit.Test;
 
 import org.apache.cassandra.cql3.CQLTester;
 import org.apache.cassandra.dht.ByteOrderedPartitioner;
+import org.apache.cassandra.exceptions.InvalidRequestException;
 import org.apache.cassandra.service.StorageService;
 
 public class UserTypesTest extends CQLTester
@@ -102,22 +103,6 @@ public class UserTypesTest extends CQLTester
         );
     }
 
-    @Test
-    public void testDescendingOrderingOfUserTypesIsSupported() throws Throwable
-    {
-        String myType = createType("CREATE TYPE %s (x double)");
-        createTable("CREATE TABLE %s (k int, v frozen<" + myType + ">, b boolean static, PRIMARY KEY (k, v)) WITH CLUSTERING ORDER BY (v DESC)");
-
-        execute("INSERT INTO %s(k, v) VALUES (?, {x:?})", 1, -104.99251);
-        execute("UPDATE %s SET b = ? WHERE k = ?", true, 1);
-
-        beforeAndAfterFlush(() ->
-            assertRows(execute("SELECT v.x FROM %s WHERE k = ? AND v = {x:?}", 1, -104.99251),
-                       row(-104.99251)
-            )
-        );
-    }
-
     @Test
     public void testInvalidUDTStatements() throws Throwable
     {
@@ -195,11 +180,6 @@ public class UserTypesTest extends CQLTester
         String myType2 = KEYSPACE + '.' + typename2;
         assertInvalidMessage("Non-frozen UDTs with nested non-frozen collections are not supported",
                 "CREATE TABLE " + KEYSPACE + ".wrong (k int PRIMARY KEY, v " + myType2 + ")");
-
-        String userType = createType("CREATE TYPE %s (userids SET<UUID>)");
-        createTable("CREATE TABLE %s (id int PRIMARY KEY)");
-        assertInvalidMessage("Non-frozen UDTs with nested non-frozen collections are not supported for column my_type",
-                             "alter TABLE %s add my_type " + userType);
     }
 
     @Test
@@ -921,4 +901,35 @@ public class UserTypesTest extends CQLTester
     {
         return keyspace() + '.' + type1;
     }
+
+    @Test
+    public void testAlteringTypeWithIfNotExits() throws Throwable
+    {
+        String columnType = typeWithKs(createType("CREATE TYPE %s (a int)"));
+
+        createTable("CREATE TABLE %s (k int PRIMARY KEY, y frozen<" + columnType + ">)");
+        execute("ALTER TYPE " + columnType + " ADD IF NOT EXISTS a int");
+
+        execute("INSERT INTO %s (k, y) VALUES(?, ?)", 1, userType("a", 1));
+        assertRows(execute("SELECT * FROM %s"), row(1, userType("a", 1)));
+
+        assertInvalidThrowMessage(String.format("Cannot add field %s to type %s: a field with name %s already exists", "a", columnType, "a"),
+                                  InvalidRequestException.class,
+                                  "ALTER TYPE " + columnType + " ADD a int");
+    }
+
+    @Test
+    public void testAlteringTypeRenameWithIfExists() throws Throwable
+    {
+        String columnType = typeWithKs(createType("CREATE TYPE %s (a int)"));
+        createTable("CREATE TABLE %s (k int PRIMARY KEY, y frozen<" + columnType + ">)");
+        execute("ALTER TYPE " + columnType + " RENAME IF EXISTS a TO z AND b TO Y;");
+
+        execute("INSERT INTO %s (k, y) VALUES(?, ?)", 1, userType("z", 1));
+        assertRows(execute("SELECT * FROM %s"), row(1, userType("z", 1)));
+
+        assertInvalidThrowMessage(String.format("Unkown field %s in user type %s", "a", columnType),
+                                  InvalidRequestException.class,
+                                  "ALTER TYPE " + columnType + " RENAME a TO z;");
+    }
 }
